Module: OpenTelemetry::SDK::Metrics::ConfiguratorPatch
- Defined in:
- lib/opentelemetry/sdk/metrics/configurator_patch.rb
Overview
The ConfiguratorPatch implements a hook to configure the metrics portion of the SDK.
Instance Method Summary collapse
-
#add_metric_reader(metric_reader) ⇒ Object
Registers a metric reader with the configured meter provider.
Instance Method Details
#add_metric_reader(metric_reader) ⇒ Object
Registers a metric reader with the configured meter provider.
14 15 16 |
# File 'lib/opentelemetry/sdk/metrics/configurator_patch.rb', line 14 def add_metric_reader(metric_reader) @metric_readers << metric_reader end |