Class: OpenTelemetry::SDK::Metrics::Exemplar::ExemplarFilter

Inherits:
Object
  • Object
show all
Defined in:
lib/opentelemetry/sdk/metrics/exemplar/exemplar_filter.rb

Overview

ExemplarFilter determines which measurements are eligible for becoming an Exemplar. Available Filters include: AlwaysOn, AlwaysOff, and TraceBased.

Class Method Summary collapse

Class Method Details

.should_sample?(value, attributes, context) ⇒ Boolean

Returns a Boolean value.

Parameters:

  • value (Integer)

    Value of the measurement

  • attributes (Hash)

    Complete set of Attributes of the measurement

  • context (Context)

    Context of the measurement, which covers the Baggage and the current active Span.

Returns:

  • (Boolean)


20
# File 'lib/opentelemetry/sdk/metrics/exemplar/exemplar_filter.rb', line 20

def self.should_sample?(value, attributes, context); end